//---------------------------------------------------------------------------------------------------------------------
// Product: EDDL
// Device : PROFIBUS PA/DP I&M Record
//          PROFIBUS Profile Guidelines: Part1 I&M Functions
// Protocol: PROFIBUS DP-V1
// (c) SIPOS Aktorik GmbH, 2013, Mathias Monse (mathias.monse@sipos.de)
//---------------------------------------------------------------------------------------------------------------------

#ifndef PA_IANDM
#define PA_IANDM

//---------------------------------------------------------------------------------------------------------------------
//~ VARIABLEs
//~ 
/*----------------------------------------------------------------------------*/
/**
 * @name MANUFACURER_ID
 * @brief The parameter MANUFACTURER_ID carries the ID for a specific manufacturer that had been
 * assigned by the PROFIBUS business office. Default value is 0000h in case a manufacturer
 * does not want to handle an ID.
 */
VARIABLE MANUFACTURER_ID
{
    LABEL       "Manufacturer"
                "|de|Hersteller"
                "|fr|Constructeur"
                "|it|Costruttore"
                "|es|Fabricante";
    HELP        "Manufacturer ID (AUMA = 319, SIPOS = 321, DREHMO = 305), see http://www.profibus.com/IM/Man_ID_Table.xml"
                "|de|Herstellerkennung (AUMA = 319, SIPOS = 321, DREHMO = 305), siehe http://www.profibus.com/IM/Man_ID_Table.xml"
                "|fr|Code du constructeur (AUMA = 319, SIPOS = 321, DREHMO = 305), http://www.profibus.com/IM/Man_ID_Table.xml"
                "|it|Codice del costruttore (AUMA = 319, SIPOS = 321, DREHMO = 305), http://www.profibus.com/IM/Man_ID_Table.xml"
                "|es|ID del fabricante (AUMA = 319, SIPOS = 321, DREHMO = 305), http://www.profibus.com/IM/Man_ID_Table.xml";
    CLASS       CONTAINED;
    TYPE        UNSIGNED_INTEGER(2)
    {
        DEFAULT_VALUE 319;
        {  42,  "SIEMENS AG"},
		{ 305,	"Drehmo"},
		{ 319,	"AUMA"},
		{ 321,	"SIPOS"}
    }
    HANDLING    READ;
}

/*----------------------------------------------------------------------------*/
/**
 * @name ORDER_ID
 * @brief This parameter contains the complete order number or at least a relevant part that allows unambiguous
 * identification of the device/module within the manufacturer's web site. In contrast
 * to [2] it comprise 20 octets. Unused octets shall be set "20h" (blank).
 */
VARIABLE ORDER_ID
{
    LABEL       "Order number"
                "|de|Bestellnummer"
                "|fr|Numro de rfrence"
                "|it|Numero di ordinazione"
                "|es|Referencia";
    HELP        "This parameter contains the complete order number or at least a relevant part that allows unambigous identification of the device/module."
                "|de|Dieser Parameter enthlt die komplette Bestellnummer oder den relevanten Teil, der eine eindeutuge Identifikation des Gerts ermglicht.";
    CLASS       CONTAINED;
    TYPE        ASCII (20);
    HANDLING    READ;
}

/*----------------------------------------------------------------------------*/
/**
 * @name SERIAL_NUMBER
 * @brief A serial number is a unique production number of the device manufacturer even for devices
 * with the same hardware, software or firmware edition. Unused characters shall be set to 20h
 * (blank).
 */
VARIABLE SERIAL_NUMBER
{
    LABEL       "Device serial number"
                "|de|Gerte Seriennummer"
                "|fr|Numro de srie de l'appareil"
                "|it|Numero di serie dispositivo"
                "|es|N de serie del aparato";
    HELP        "Unique module serial number"
                "|de|Eindeutige identifizierbare Seriennummer der Baugruppe"
                "|fr|Numro de srie servant  l'identification univoque du module"
                "|it|Numero di serie univoco dell'unit"
                "|es|N de serie unvoco del mdulo";
    CLASS       CONTAINED;
    TYPE        ASCII (16)
    {
        DEFAULT_VALUE "00";
    }
    HANDLING    READ;
}

/*----------------------------------------------------------------------------*/
/**
 * @name HARDWARE_REVISION
 * @brief The content of this parameter characterizes the edition of the hardware only. Default value is
 * 0000h. FFFFh indicates availability of profile specific information. Hint: individual revision information
 * may be stored within the manufacturer specific parts of the I&M functions. However,
 * this information can only be made available via EDD or DTM.
 */
VARIABLE HARDWARE_REVISION
{
    LABEL       "Hardware revision"
                "|de|Hardware-Revision"
                "|fr|Rvision matrielle"
                "|it|Versione hardware"
                "|es|Revisin del hardware";
    HELP        "Hardware version"
                "|de|Hardware-Ausgabestand"
                "|fr|Version du matriel"
                "|it|Versione hardware"
                "|es|Versin de hardware";
    CLASS       CONTAINED;
    TYPE        UNSIGNED_INTEGER(2);
    HANDLING    READ;
}

/*----------------------------------------------------------------------------*/
/**
 * @name SOFTWARE_REVISION
 * @brief The content of this parameter characterizes the edition of the hardware only. Default value is
 * 0000h. FFFFh indicates availability of profile specific information. Hint: individual revision information
 * may be stored within the manufacturer specific parts of the I&M functions. However,
 * this information can only be made available via EDD or DTM.
 */
VARIABLE SOFTWARE_REVISION_byte0
{
    LABEL       "Software Revision, byte 0";
    CLASS       CONTAINED;
    TYPE        ASCII(1)
    {
        DEFAULT_VALUE "V";
    }
    HANDLING    READ;
    POST_READ_ACTIONS
    {
        DEFINITION
        {
            SOFTWARE_REVISION = SOFTWARE_REVISION_byte0        + 
                                itoa(SOFTWARE_REVISION_byte1)  + "." +
                                itoa(SOFTWARE_REVISION_byte2)  + "." +
                                itoa(SOFTWARE_REVISION_byte3);
			SOFTWARE_REVISION.VARIABLE_STATUS= SOFTWARE_REVISION_byte0.VARIABLE_STATUS;
			save_values();
        }
    }    
}

VARIABLE SOFTWARE_REVISION_byte1
{
    LABEL       "Software Revision, byte 1";
    CLASS       CONTAINED;
    TYPE        UNSIGNED_INTEGER(1);
    HANDLING    READ;
}

SOFTWARE_REVISION_byte2    LIKE VARIABLE SOFTWARE_REVISION_byte1 {}
SOFTWARE_REVISION_byte3    LIKE VARIABLE SOFTWARE_REVISION_byte1 {}

VARIABLE SOFTWARE_REVISION
{
    LABEL       "Software Revision"
                "|de|Software-Revision"
                "|fr|Rvision logicielle"
                "|it|Versione software"
                "|es|Revisin de software";
    HELP        "Software Release"
                "|de|Software-Ausgabestand"
                "|fr|Version du logiciel"
                "|it|Versione software"
                "|es|Versin del software";
    CLASS       LOCAL;
    TYPE        ASCII(12);
    HANDLING    READ;
}

/*----------------------------------------------------------------------------*/
/**
 * @name REVISION_COUNTER
 * @brief A changed value of the REV_COUNTER parameter of a given module marks a change of
 * hardware or of its parameters. At production time the counter will be set to 0. This value is
 * reserved for the first installation and the first increment. Thus, the counter increments from
 * 1... 65535, wrapping over back to 1 at the end.
 * Criteria to increment the counter:
 *   - Module exchange
 *   - Any write access of parameters within the module and its activation
 */
VARIABLE REVISION_COUNTER
{
    LABEL       "Static Revision No."
                "|de|Statische Revisions-Nr."
                "|fr|No de rvision statique"
                "|it|Numero di revisione statica"
                "|es|N de revisin esttico";
    HELP        "Information about configured module modifications"
                "|de|Auskunft ber parametrierte nderungen des Moduls"
                "|fr|Informations sur les modifications paramtres du module"
                "|it|Informazioni sulle modifiche parametrate del modulo"
                "|es|Informacin sobre cambios parametrizados del mdulo";
    CLASS       CONTAINED;
    TYPE        UNSIGNED_INTEGER(2);
    HANDLING    READ;
}

/*----------------------------------------------------------------------------*/
/**
 * @name PROFILE_ID
 * @brief A module following a special profile may offer extended information (PROFILE_
 * SPECIFIC_TYPE) about its function and/or sub devices, e.g. HART. Devices not following
 * any PROFIBUS application profile shall use F600h. In this case the device is generic and
 * may indicate its type via the parameter PROFILE_SPECIFIC_TYPE.
 */
VARIABLE PROFILE_ID
{
    LABEL       "Profile"
                "|de|Profil";
    CLASS       CONTAINED;
    TYPE        UNSIGNED_INTEGER(2);
    HANDLING    READ;
    POST_READ_ACTIONS
    {
        DEFINITION
        {
            if ((PROFILE_ID >= 0x3A00) && (PROFILE_ID <= 0x3AFF))
            {
                PROFILE_ID_display = 0;
            }
            else if ((PROFILE_ID >= 0x3B00) && (PROFILE_ID <= 0x3BFF))
            {
                PROFILE_ID_display = 1;
            }
            else if ((PROFILE_ID >= 0x3C00) && (PROFILE_ID <= 0x3CFF))
            {
                PROFILE_ID_display = 2;
            }
            else if ((PROFILE_ID >= 0x3D00) && (PROFILE_ID <= 0x3DFF))
            {
                PROFILE_ID_display = 3;
            }
            else if ((PROFILE_ID >= 0x3E00) && (PROFILE_ID <= 0x3EFF))
            {
                PROFILE_ID_display = 4;
            }
            else if ((PROFILE_ID >= 0x3F00) && (PROFILE_ID <= 0x3FFF))
            {
                PROFILE_ID_display = 5;
            }
            else if ((PROFILE_ID >= 0x4900) && (PROFILE_ID <= 0x49FF))
            {
                PROFILE_ID_display = 6;
            }
            else if ((PROFILE_ID >= 0x4B00) && (PROFILE_ID <= 0x4BFF))
            {
                PROFILE_ID_display = 7;
            }
            else if ((PROFILE_ID >= 0x4C00) && (PROFILE_ID <= 0x4CFF))
            {
                PROFILE_ID_display = 8;
            }
            else if ((PROFILE_ID >= 0x4E00) && (PROFILE_ID <= 0x4EFF))
            {
                PROFILE_ID_display = 9;
            }
            else if ((PROFILE_ID >= 0x4F00) && (PROFILE_ID <= 0x4FFF))
            {
                PROFILE_ID_display = 10;
            }
            else if ((PROFILE_ID >= 0x5A00) && (PROFILE_ID <= 0x5AFF))
            {
                PROFILE_ID_display = 11;
            }
            else if ((PROFILE_ID >= 0x5B00) && (PROFILE_ID <= 0x5BFF))
            {
                PROFILE_ID_display = 12;
            }
            else if ((PROFILE_ID >= 0x5C00) && (PROFILE_ID <= 0x5CFF))
            {
                PROFILE_ID_display = 13;
            }
            else if ((PROFILE_ID >= 0x5D00) && (PROFILE_ID <= 0x5DFF))
            {
                PROFILE_ID_display = 14;
            }
            else if ((PROFILE_ID >= 0x5E00) && (PROFILE_ID <= 0x5EFF))
            {
                PROFILE_ID_display = 15;
            }
            else if ((PROFILE_ID >= 0x5F00) && (PROFILE_ID <= 0x5FFF))
            {
                PROFILE_ID_display = 16;
            }
            else if ((PROFILE_ID >= 0x6200) && (PROFILE_ID <= 0x62FF))
            {
                PROFILE_ID_display = 17;
            }
            else if ((PROFILE_ID >= 0x6A00) && (PROFILE_ID <= 0x6AFF))
            {
                PROFILE_ID_display = 18;
            }
            else if ((PROFILE_ID >= 0x6B00) && (PROFILE_ID <= 0x6BFF))
            {
                PROFILE_ID_display = 19;
            }
            else if ((PROFILE_ID >= 0x6C00) && (PROFILE_ID <= 0x6CFF))
            {
                PROFILE_ID_display = 20;
            }
            else if ((PROFILE_ID >= 0x6D00) && (PROFILE_ID <= 0x6DFF))
            {
                PROFILE_ID_display = 21;
            }
            else if ((PROFILE_ID >= 0x6E00) && (PROFILE_ID <= 0x6EFF))
            {
                PROFILE_ID_display = 22;
            }
            else if ((PROFILE_ID >= 0x6F00) && (PROFILE_ID <= 0x6FFF))
            {
                PROFILE_ID_display = 23;
            }
            else if ((PROFILE_ID >= 0x9700) && (PROFILE_ID <= 0x97FF))
            {
                PROFILE_ID_display = 24;
            }
            else if ((PROFILE_ID >= 0xF600) && (PROFILE_ID <= 0xF6FF))
            {
                PROFILE_ID_display = 25;
            }
            else
            {
                PROFILE_ID_display = 99;
            }
			PROFILE_ID_display.VARIABLE_STATUS= PROFILE_ID.VARIABLE_STATUS;
			save_values();
        }
    }
}

VARIABLE PROFILE_ID_display
{
    LABEL       "Profile"
                "|de|Profil";
    CLASS       LOCAL;
    TYPE        ENUMERATED(1)
    {
        {   0,  "PROFIdrive"                        },
        {   1,  "Robots and Numeric Controls"       },
        {   2,  "Display & Panel Devices"           },
        {   3,  "Encoder"                           },
        {   4,  "PROFIsafe"                         },
        {   5,  "Fluid Power / Hydraulics"          },
        {   6,  "Water Treatments"                  },
        {   7,  "Reserved"                          },
        {   8,  "Reserved"                          },
        {   9,  "Reserved"                          },
        {  10,  "Reserved"                          },
        {  11,  "SEMI - Devices"                    },
        {  12,  "Identification Systems"            },
        {  13,  "Weighing & Dosage Systems"         },
        {  14,  "Intelligent Pumps"                 },
        {  15,  "Low Voltage Switchgear"            },
        {  16,  "Remote I/O for Process Control"    },
        {  17,  "Laboratory Devices"                },
        {  18,  "Reserved"                          },
        {  19,  "Reserved"                          },
        {  20,  "Reserved"                          },
        {  21,  "Reserved"                          },
        {  22,  "Reserved"                          },
        {  23,  "Reserved"                          },
        {  24,  "PA Devices (for Process Control)"  },
        {  25,  "Generic device"                    },
        {  99,  "not defined"                       }
    }
    HANDLING    READ;
}

/*----------------------------------------------------------------------------*/
/**
 * @name PROFILE_SPECIFIC_TYPE
 * @brief In case a module follows a special profile (see table in chapter 3.2.8) this parameter offers
 * information about the usage of its channels and/or sub devices. The module/channel information
 * shall be according to the respective definitions of the application profile. Devices/modules
 * not following any PROFIBUS application profile are generic and shall use F600h  F6FFh. In
 * this case the parameter indicates the type of sensor or actuator ("device type"; coding defined
 * in Appendix 7.1).
 */
VARIABLE PROFILE_SPECIFIC_TYPE
{
    LABEL       "Device class"
                "|de|Gerteklasse";
    CLASS       CONTAINED;
    TYPE        UNSIGNED_INTEGER(2);
    HANDLING    READ;
}

/*----------------------------------------------------------------------------*/
/**
 * @name IM_VERSION
 * @brief This parameter indicates the implemented version of the I&M functions.
 */
VARIABLE IM_VERSION
{
    LABEL       "I&M Version";
    HELP        "Implemented version  of the I&M functions."
                "|de|Implementierte Version der I&M Funktionen.";
    CLASS       LOCAL;
    TYPE        ASCII(7);
    HANDLING    READ;    
}

VARIABLE IM_VERSION_HB
{
    LABEL       "I&M Version";
    CLASS       CONTAINED;
    TYPE        UNSIGNED_INTEGER(1);
    HANDLING    READ;
}

VARIABLE IM_VERSION_LB
{
    LABEL       "I&M Version";
    CLASS       CONTAINED;
    TYPE        UNSIGNED_INTEGER(1);
    HANDLING    READ;
    POST_READ_ACTIONS
    {
        DEFINITION
        {
            IM_VERSION = itoa(IM_VERSION_HB) + "." + itoa(IM_VERSION_LB);
			IM_VERSION.VARIABLE_STATUS= IM_VERSION_HB.VARIABLE_STATUS;
			save_values();
        }
    }        
}

/*----------------------------------------------------------------------------*/
/**
 * @name IM_SUPPORTED
 * @brief This parameter indicates the availability of I&M records.
 */
VARIABLE IM_SUPPORTED
{
    LABEL       "Available I&M records"
                "|de|Verfgbare I&M Daten";
    HELP        "This parameter indicates the availability of I&M records (bit 0 <-> I&M 0, bit 1 <-> I&M 1 (opt.), ...)."
                "|de|Der Parameter zeigt verfgbare I&M Daten-Blcke an (Bit 0 <-> I&M 0, Bit 1 <-> I&M 1 (opt.), ...).";
    CLASS       CONTAINED;
    TYPE        UNSIGNED_INTEGER(2);
    HANDLING    READ;
}

VARIABLE TAG_FUNCTION
{
    LABEL       "TAG"
                "|de|TAG"
                "|fr|Repre de l'appareil"
                "|it|TAG"
                "|es|TAG";
    HELP        "TAG: can be used in any way. A recommended use is as a unique label for a field device in the plant."
                "|de|TAG: Betriebsmittelkennzeichen, das frei verwendet werden kann. Es wird empfohlen, eine eindeutige Kennzeichnung fr das Feldgert in der Anlage zu vergeben."
                "|fr|Repre d'identification dans l'installation : nom univoque donn par l'utilisateur  l'appareil."
                "|it|Contrassegno dei dispositivi che pu essere usato liberamente. Si raccomanda di assegnare una denominazione univoca per l'apparecchio di campo nell'impianto."
                "|es|Identificador de los medios de produccin que se puede utilizar libremente. Es aconsejable asignar un identificador unvoco al aparato de campo dentro de la instalacin.";
    CLASS       LOCAL;
    TYPE        ASCII (32)
    {
        DEFAULT_VALUE   "Add FEM HART";
    }
    HANDLING    READ & WRITE;
}


VARIABLE TAG_LOCATION
{
    LABEL       "Tag location"
                "|de|Einbauort";
    HELP        "Tag location of the device. Max. 16 characters."
                "|de|Einbauort, an dem die Baugruppe eingebaut ist. Maximal 22 Zeichen.";
    CLASS       LOCAL; /* wenn im Gert verfgbar: CONTAINED; */
    TYPE        ASCII (22);
    HANDLING    READ & WRITE;
}

VARIABLE INSTALLATION_DATE
{
    LABEL       "Installation date"
                "|de|Einbaudatum"
                "|fr|Date d'installation"
                "|it|Data di installazione"
                "|es|Fecha de montaje";
    HELP        "Date on which the module was installed. Max. 16 characters."
                "|de|Datum, an dem die Baugruppe eingebaut wurde. Maximal 16 Zeichen."
                "|fr|Date  laquelle le module a t install. 16 caractres max."
                "|it|Data in cui l'unit  stata installata l'unit. Max. 16 caratteri."
                "|es|Fecha de montaje del mdulo16 caracteres como mximo";
    CLASS       LOCAL; /* wenn im Gert verfgbar: CONTAINED; */
    TYPE        ASCII(16);
    HANDLING    READ & WRITE;
}

VARIABLE DESCRIPTOR
{
    LABEL       "Description"
                "|de|Beschreibung"
                "|fr|Description"
                "|it|Descrizione"
                "|es|Descripcin";
    HELP        "Additional information, Individual comments, max. 54 characters."
                "|de|Zusatzinformationen, Individueller Kommentar, maximal 54 Zeichen eingebbar."
                "|fr|Informations complmentaires, commentaire individuel, 54 caractres max."
                "|it|Informazioni supplementari e commenti, max.54 caratteri."
                "|es|Informacin adicional, comentario individual, 54 caracteres como mximo.";
    CLASS       LOCAL; /* wenn im Gert verfgbar: CONTAINED; */
    TYPE        ASCII(54);
    HANDLING    READ & WRITE;
}

VARIABLE SIGNATURE
{
    LABEL       "Signature"
                "|de|Signatur";
    HELP        "Reference for specific parameter settings"
                "|de|Referenz fr eine bestimmte Parametrierung, maximal 54 Zeichen eingebbar.";
    CLASS       LOCAL; /* wenn im Gert verfgbar: CONTAINED; */
    TYPE        ASCII(54);
    HANDLING    READ & WRITE;
}

/*----------------------------------------------------------------------------*/
/**
 * @name IM_req_extfun
 * @brief CALL_HEADER::EXTENDED_FUNCTION_NUM
 */
VARIABLE IM_req_extfun
{
    LABEL       "IM_req_extfun";
    CLASS       CONTAINED;
    TYPE        UNSIGNED_INTEGER(1)
	{
		DEFAULT_VALUE    0x08;
	}
    HANDLING    WRITE;
}
IM_rsp_extfun LIKE VARIABLE IM_req_extfun { REDEFINE LABEL "IM_rsp_extfun"; REDEFINE HANDLING READ; }
/*----------------------------------------------------------------------------*/
/**
 * @name IM_req_index
 * @brief CALL_HEADER::IM_INDEX
 */
VARIABLE IM_req_index
{
    LABEL       "IM_req_index";
    CLASS       CONTAINED;
    TYPE        UNSIGNED_INTEGER(2)
	{
		DEFAULT_VALUE	65000;
	}
    HANDLING    WRITE;
}
IM_rsp_index LIKE VARIABLE IM_req_index { REDEFINE LABEL "IM_rsp_index"; REDEFINE HANDLING READ; }

//---------------------------------------------------------------------------------------------------------------------
//~ COMMANDs
//~ 
/*----------------------------------------------------------------------------*/
COMMAND  write_IM0_record_REQ
{
	SLOT 0;
	INDEX 255;
	OPERATION WRITE;
	TRANSACTION
	{
		REQUEST
		{
			IM_req_extfun,
			0x00,
			IM_req_index	
		}
		REPLY
		{
		}
	}
}

/*----------------------------------------------------------------------------*/
COMMAND  read_IM0_record_RSP
{
	SLOT 0;
	INDEX 255;
	OPERATION READ;
	TRANSACTION
	{
		REQUEST
		{
		}
		REPLY
		{
			IM_rsp_extfun,
			0x00,
			IM_rsp_index,
			0x00,0x00,0x00,0x00,0x00,
			0x00,0x00,0x00,0x00,0x00,
            MANUFACTURER_ID,
            ORDER_ID,
            SERIAL_NUMBER,
            HARDWARE_REVISION,
            SOFTWARE_REVISION_byte0,
            SOFTWARE_REVISION_byte1,
            SOFTWARE_REVISION_byte2,
            SOFTWARE_REVISION_byte3,
            REVISION_COUNTER,
            PROFILE_ID,
            PROFILE_SPECIFIC_TYPE,
            IM_VERSION_LB,
            IM_VERSION_HB,
            IM_SUPPORTED
		}
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD
// 
METHOD  refresh_IM_record
{
	LABEL	"refresh_IM_record";
	DEFINITION
	{
		// EXTENDED_FUNCTION_NUM= "CALL"
		IM_req_extfun= 0x08;
		// I&M 0 record
		IM_req_index= 65000;
		WriteCommand(write_IM0_record_REQ);
		ReadCommand(read_IM0_record_RSP);
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   MENU
// 
MENU menu_IM_record
{
    LABEL		"Identification and Maintenance (I+M 0)"
	            "|de|I+M-Informationen"; // [TXT_menu_IM_record]; 
	STYLE		PAGE;
	ITEMS
	{
		MANUFACTURER_ID,
		COLUMNBREAK,
		ORDER_ID,
		COLUMNBREAK,
		SERIAL_NUMBER,
		ROWBREAK,
		HARDWARE_REVISION,
		COLUMNBREAK,
		SOFTWARE_REVISION,
		COLUMNBREAK,
		REVISION_COUNTER,
		ROWBREAK,
		PROFILE_ID_display,
		COLUMNBREAK,
		PROFILE_SPECIFIC_TYPE,
		COLUMNBREAK,
		IM_VERSION,
		ROWBREAK,
		IM_SUPPORTED (HIDDEN)
	}
}
//---------------------------------------------------------------------------------------------------------------------
//~  EOF
//---------------------------------------------------------------------------------------------------------------------
#endif